"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLatestNodeVersion = exports.updatePackageJson = exports.removePackages = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const constants_1 = require("./constants");
const https_1 = require("https");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
function removePackages(name) {
    return (tree, _context) => {
        const path = `/${name}/package.json`;
        const json = readJsonFile(tree, path);
        if (json) {
            // Update scripts
            json.scripts = Object.assign(Object.assign({}, json.scripts), { test: 'jest' });
            // Add static dependencies
            json.dependencies = Object.assign(Object.assign({}, json.dependencies), constants_1.staticDependencies);
            // Add new devDependencies
            json.devDependencies = Object.assign(Object.assign({}, json.devDependencies), constants_1.devDependencies);
            // Remove dependencies
            constants_1.removeDependencies.forEach((dependency) => delete json.devDependencies[dependency]);
            tree.overwrite(path, JSON.stringify(json, null, 2));
            return tree;
        }
        throw new schematics_1.SchematicsException(`Does not exist ${path}.`);
    };
}
exports.removePackages = removePackages;
function updatePackageJson(name) {
    return (0, schematics_1.chain)([removePackages(name), addPackages(name)]);
}
exports.updatePackageJson = updatePackageJson;
function addPackages(name) {
    const path = `/${name}/package.json`;
    return (tree, context) => {
        const newDependencies = (0, rxjs_1.of)(...constants_1.dependencies).pipe((0, operators_1.concatMap)((packageName) => getLatestNodeVersion(packageName)), (0, operators_1.map)((packageFromRegistry) => {
            const { name, version } = packageFromRegistry;
            context.logger.debug(`Adding ${name}:${version} to Dev Dependencies`);
            addPackageJsonDependency(tree, name, version, path);
            return tree;
        }));
        return (0, rxjs_1.concat)(newDependencies);
    };
}
/**
 * Attempt to retrieve the latest package version from NPM
 * Return an optional "latest" version in case of error
 * @param packageName
 */
function getLatestNodeVersion(packageName) {
    const DEFAULT_VERSION = 'latest';
    return new Promise((resolve) => {
        return (0, https_1.get)(`https://registry.npmjs.org/${packageName}`, (res) => {
            let rawData = '';
            res.setEncoding('utf8');
            res.on('data', (chunk) => {
                rawData += chunk;
            });
            res.on('end', () => {
                try {
                    const response = JSON.parse(rawData);
                    const version = (response && response['dist-tags']) || {};
                    resolve(buildPackage(packageName, version.latest));
                }
                catch (e) {
                    resolve(buildPackage(packageName));
                }
            });
        }).on('error', () => resolve(buildPackage(packageName)));
    });
    function buildPackage(name, version = DEFAULT_VERSION) {
        return { name, version };
    }
}
exports.getLatestNodeVersion = getLatestNodeVersion;
function readJsonFile(tree, path) {
    if (tree.exists(path)) {
        const file = tree.read(path);
        return JSON.parse(file.toString());
    }
    return null;
}
function addPackageJsonDependency(tree, name, version, path) {
    const json = readJsonFile(tree, path);
    if (json) {
        json.dependencies = Object.assign(Object.assign({}, json.dependencies), { [name]: `^${version}` });
    }
    tree.overwrite(path, JSON.stringify(json, null, 2));
    return tree;
}
//# sourceMappingURL=index.js.map